/* GtkBalls
 * Copyright (C) 1998-1999 Eugene V. Morozov
 * Modifyed in 2001 by drF_ckoff
 * $Id: interface.c,v 1.3 1999/12/28 03:33:35 jmv Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <config.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <unistd.h>

#include "gtkballs.h"
#include "interface.h"
#include "scoreboard.h"
#include "preferences.h"
#include "path.h"

GdkPixmap *Pixmap=NULL;

gbPixmap gbSmallBalls[NUMBER_OF_BALLS+1];
gbPixmap gbBalls[NUMBER_OF_BALLS+1];
gbPixmap gbPaws[8];

/*default sizes*/
gint CellXSize;
gint CellYSize;

/* Three balls that shows what colors will appear on next turn
 * I use this variables in preferences.[ch] */
GtkWidget *Ball1,*Ball2,*Ball3;

/* Current size of the board
 * Also used in interface.c */
gint NumberOfXCells=NUMBER_OF_CELLS;
gint NumberOfYCells=NUMBER_OF_CELLS;

/* Number and color of the jumping ball */
gint Jumping_ball=0;
gint Jumping_ball_color=0;

gint AnimationInProgress=FALSE;

gint PawX=-1,PawY=-1;

guint TimerTag;

/* Scoreboard is also used in interface.c */
struct score_board Scoreboard[10];

/*some file/dir names*/
gchar *installpath=DATADIR "/gtkballs/themes/";
gchar *themeprefix="/.gtkballs/themes/";

gchar *find_theme_path(gchar *themename) {
  	gchar *homedir;
  	gchar *themepath;
  	struct stat buf;

  	if((homedir=getenv("HOME")) && (themepath=g_strconcat(homedir,themeprefix,themename,NULL))) {
  		if(!stat(themepath,&buf) && S_ISDIR(buf.st_mode)) {
			return themepath;
        	}
        	g_free(themepath);
	}
  	if((themepath=g_strconcat(installpath,themename,NULL))) {
        	if(!stat(themepath,&buf) && S_ISDIR(buf.st_mode)) {
			return themepath;
        	}
        	g_free(themepath);
  	}
  	return NULL;
}

gint gb_load_pixmap(gbPixmap *pixmap, gchar *themepath, gchar *pixmapname) {
  	gchar *item;

  	if(!(item=g_strconcat(themepath,"/",pixmapname,NULL))) return 0;
  	if(pixmap->pixmap) gdk_pixmap_unref(pixmap->pixmap);
  	if(pixmap->mask) gdk_bitmap_unref(pixmap->mask);
  	if(!(pixmap->pixmap=gdk_pixmap_create_from_xpm(main_window->window, &pixmap->mask,&style->bg[GTK_STATE_NORMAL], item))) {
        	g_free(item);
        	perror(item);
        	return 0;
  	}
  	gdk_window_get_size((GdkWindow *)pixmap->pixmap,&pixmap->xsize,&pixmap->ysize);
  	g_free(item);
  	return 1;
}

gint load_theme(gchar *themename) {
  	gchar *themepath;
  	gint  i;
  	gchar *ballnames[]={"empty_cell.xpm","blue_ball.xpm","brown_ball.xpm","green_ball.xpm","red_ball.xpm","yellow_ball.xpm","pink_ball.xpm","cyan_ball.xpm"};
  	gchar *smallballnames[]={"small_empty_cell.xpm","small_blue_ball.xpm","small_brown_ball.xpm","small_green_ball.xpm","small_red_ball.xpm","small_yellow_ball.xpm","small_pink_ball.xpm","small_cyan_ball.xpm"};
  	gchar *pawnames[]={"paw_down.xpm","paw_left.xpm","paw_up.xpm","paw_right.xpm","paw_down_left.xpm","paw_down_right.xpm","paw_up_left.xpm","paw_up_right.xpm"};

  	if(!(themepath=find_theme_path(themename))) return 0;
  	for(i=0;i<8;i++) {
        	if(!(gb_load_pixmap(&gbBalls[i], themepath, ballnames[i]))) {
                	g_free(themepath);
                	return 0;
        	}
        	if(!i) {
                	CellXSize=gbBalls[i].xsize;
                	CellYSize=gbBalls[i].ysize;
        	}
        	if(!(gb_load_pixmap(&gbSmallBalls[i], themepath, smallballnames[i]))) {
                	g_free(themepath);
	                return 0;
        	}
	        if(!(gb_load_pixmap(&gbPaws[i], themepath, pawnames[i]))) {
        	        g_free(themepath);
                	return 0;
	        }
	}
  	g_free(themepath);
  	gtk_drawing_area_size(GTK_DRAWING_AREA(DrawingArea), NumberOfXCells*CellXSize, NumberOfYCells*CellYSize);
  	if(Pixmap) {
		gdk_pixmap_unref(Pixmap);
  		Pixmap=gdk_pixmap_new(DrawingArea->window, NumberOfXCells*CellXSize, NumberOfYCells*CellYSize, -1);
  	}
  	return 1;
}

void draw_next_balls(void) {
	gtk_pixmap_set(GTK_PIXMAP(Ball1), gbSmallBalls[NextColors[0]].pixmap, gbSmallBalls[NextColors[0]].mask);
	gtk_pixmap_set(GTK_PIXMAP(Ball2), gbSmallBalls[NextColors[1]].pixmap, gbSmallBalls[NextColors[1]].mask);
      	gtk_pixmap_set(GTK_PIXMAP(Ball3), gbSmallBalls[NextColors[2]].pixmap, gbSmallBalls[NextColors[2]].mask);
}

void draw_ball_no_update(GtkWidget *widget, gint ballcolor, GdkPixmap *pixmap, gint x, gint y, gint yoffset) {
        gint xr=x*CellXSize+(CellXSize-gbBalls[ballcolor].xsize)/2;
        gint yr=y*CellYSize+(CellYSize-gbBalls[ballcolor].ysize)/2+yoffset-5;

  	gdk_draw_pixmap(pixmap, widget->style->fg_gc[GTK_WIDGET_STATE (widget)], gbBalls[0].pixmap,
		   	0, 0, x*CellXSize, y*CellYSize, CellXSize, CellYSize);
  	if(ballcolor) {
        	gdk_gc_set_clip_mask(widget->style->fg_gc[GTK_WIDGET_STATE(widget)], gbBalls[ballcolor].mask);
        	gdk_gc_set_clip_origin(widget->style->fg_gc[GTK_WIDGET_STATE(widget)], xr, yr);
        	gdk_draw_pixmap(pixmap, widget->style->fg_gc[GTK_WIDGET_STATE(widget)], gbBalls[ballcolor].pixmap,
				0, 0, xr, yr, gbBalls[ballcolor].xsize, gbBalls[ballcolor].ysize);
        	gdk_gc_set_clip_origin(widget->style->fg_gc[GTK_WIDGET_STATE(widget)], 0, 0);
        	gdk_gc_set_clip_mask(widget->style->fg_gc[GTK_WIDGET_STATE(widget)], 0);
  	}
}

void draw_ball(GtkWidget *widget, gint ballcolor, GdkPixmap *pixmap, gint x, gint y, gint yoffset) {
  	draw_ball_no_update(widget, ballcolor, pixmap, x, y, yoffset);
  	update_rectangle(widget, x*CellXSize, y*CellYSize, CellXSize, CellYSize);
}

void draw_paw(GtkWidget *widget, gint pawnumber, GdkPixmap *pixmap, gint x, gint y) {
        gint xr=x*CellXSize+(CellXSize-gbPaws[pawnumber].xsize)/2;
        gint yr=y*CellYSize+(CellYSize-gbPaws[pawnumber].ysize)/2;

  	gdk_draw_pixmap(pixmap, widget->style->fg_gc[GTK_WIDGET_STATE(widget)], gbBalls[0].pixmap,
		   	0, 0, x*CellXSize, y*CellYSize, CellXSize, CellYSize);
  	gdk_gc_set_clip_mask(widget->style->fg_gc[GTK_WIDGET_STATE(widget)], gbPaws[pawnumber].mask);
  	gdk_gc_set_clip_origin(widget->style->fg_gc[GTK_WIDGET_STATE(widget)], xr, yr);
  	gdk_draw_pixmap(pixmap, widget->style->fg_gc[GTK_WIDGET_STATE(widget)], gbPaws[pawnumber].pixmap,
			0, 0, xr, yr, gbPaws[pawnumber].xsize, gbPaws[pawnumber].ysize);
  	gdk_gc_set_clip_origin(widget->style->fg_gc[GTK_WIDGET_STATE(widget)], 0, 0);
  	gdk_gc_set_clip_mask(widget->style->fg_gc[GTK_WIDGET_STATE(widget)], 0);
  	update_rectangle(widget, x*CellXSize, y*CellYSize, CellXSize, CellYSize);
}

void draw_board(GtkWidget *widget, GdkPixmap *pixmap) {
  	gint i, j;

  	for(i=0;i<NumberOfXCells;i++) {
    		for(j=0;j<NumberOfYCells;j++) {
      			if(!AnimationInProgress || (((j*NumberOfXCells+i)!=Jumping_ball) && (PawX!=i) && (PawY!=j))) {
              			draw_ball_no_update (widget, Board[i][j], pixmap, i, j, 5);
                        }
                }
        }
}

/* Refill the screen from the backing pixmap */
gint expose_event(GtkWidget *widget, GdkEventExpose *event) {
  	draw_board(widget, Pixmap);
  	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)], Pixmap,
		   	event->area.x, event->area.y, event->area.x, event->area.y,
			event->area.width, event->area.height);
  	return FALSE;
}

void find_pawnum_and_direction(gint PawX, gint PawY, gint x, gint y, gint *pawnum, gint *direction) {
	if(PawY<y) {
        	*pawnum=2;
                if(*direction==1) *pawnum=6;
        	if(*direction==3) *pawnum=7;
        	*direction=2; /* up */
	} else if(PawY>y) {
        	*pawnum=0;
                if(*direction==1) *pawnum=4;
                if(*direction==3) *pawnum=5;
	        *direction=0; /* down */
	} else if(PawX<x) {
        	*pawnum=1;
                if(*direction==0) *pawnum=4;
                if(*direction==2) *pawnum=6;
	        *direction=1; /* left */
	} else if(PawX>x) {
        	*pawnum=3;
                if(*direction==0) *pawnum=5;
                if(*direction==2) *pawnum=7;
	        *direction=3; /* right */
        }
}

gint find_direction(gint PawX, gint PawY, gint x, gint y) {
	if(PawY<y) return 2; /* up */
	if(PawY>y) return 0; /* down */
        if(PawX<x) return 1; /* left */
        if(PawX>x) return 3; /* right */
        return -1; /* should never happens */
}

gint move_ball(GtkWidget *widget, gint source_ball, gint target_ball) {
  	gint nodes[NumberOfXCells*NumberOfYCells];
  	gint path[NumberOfXCells*NumberOfYCells];
  	gint i,j,k,phase;
  	gint x,y,color,tbx,tby;
  	gint direction,pawnum;

  	find_x_y_of_the_node(&tbx, &tby, target_ball, NumberOfXCells, NumberOfYCells);
  	if(Board[tbx][tby]) return 2;

  	for(i=0;i<NumberOfXCells;i++) {
    		for(j=0;j<NumberOfYCells;j++) {
      			if(Board[i][j]!=0) {
				nodes[j*NumberOfXCells+i]=-1;
      			} else {
				nodes[j*NumberOfYCells+i]=0;
                        }
        	}
        }

  	nodes[source_ball]=nodes[target_ball]=0;
  	if(!find_path(nodes, source_ball, target_ball, path, NumberOfXCells, NumberOfYCells)) {
		return 0;
        }

  	find_x_y_of_the_node(&x, &y, source_ball, NumberOfXCells, NumberOfYCells);
  	color=Board[x][y];
  	Board[x][y]=0;
  	phase=0;

  	if(Show_path){
      		AnimationInProgress=TRUE;
      		draw_ball(widget, 0, Pixmap, x, y, 6);

      		for(k=path[0]-1;k;k--) {
	  		find_x_y_of_the_node(&i, &j, path[k], NumberOfXCells, NumberOfYCells);
	  		Jumping_ball=j*NUMBER_OF_CELLS+i;
	  		if(k==path[0]-1) {
	      			/* x and y are the coordinates of starting position */
	      			PawX = x;
	      			PawY = y;
	  		} else {
	      			find_x_y_of_the_node(&PawX, &PawY, path[k+1], NumberOfXCells, NumberOfYCells);
	  		}
          		if(k!=path[0]-1) {
                                find_pawnum_and_direction(PawX, PawY, i, j, &pawnum, &direction);
          		} else {
                		pawnum=direction=find_direction(PawX, PawY, i, j);
          		}
	  		if(Show_footprints) {
	    			draw_paw (widget, pawnum, Pixmap, PawX, PawY);
                        }
	  		for(phase=0;phase<NUMBER_OF_PHASES;phase++) {
	      			draw_ball(widget, Jumping_ball_color, Pixmap, i, j,
			 		  (phase>NUMBER_OF_PHASES/2) ? phase+1 : 3-phase);
	      			usleep(10000);
	  		}
          		draw_ball(widget, 0, Pixmap, i, j, 5);
      		}
      		Jumping_ball = tby*NUMBER_OF_CELLS+tbx;
      		if(k==path[0]-1) {
	  		/* x and y are the coordinates of starting position */
	  		PawX=x;
	  		PawY=y;
      		} else {
	  		find_x_y_of_the_node(&PawX, &PawY, path[k+1], NumberOfXCells, NumberOfYCells);
      		}
      		if (Show_footprints) {
                	find_pawnum_and_direction(PawX, PawY, tbx, tby, &pawnum, &direction);
			draw_paw (widget, pawnum, Pixmap, PawX, PawY);
                }
  	}
        PawX=-1;
        PawY=-1;
  	AnimationInProgress=FALSE;
  	Board[tbx][tby]=color;
  	draw_ball(widget, color, Pixmap, tbx, tby, 6);
  	return 1;
}

