/* GtkBalls
 * Copyright (C) 1998-1999 Eugene V. Morozov
 * Modifyed in 2001 by drF_ckoff
 * $Id: gtkballs.c,v 1.3 1999/12/28 03:33:25 jmv Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <locale.h>
#include <sys/param.h>
#include <time.h>
#include <gtk/gtk.h>
#include <X11/Xlocale.h>

#include "gtkballs.h"
#include "interface.h"
#include "preferences.h"
#include "scoreboard.h"
#include "path.h"
#include "license.h"
#include "gfx.h"

/* ----- Global variables ----- */

/* Note that I'm trying to start global variables' names from
 * uppercase letter whenever possible */

/* The board itself
 * Also used in interface.[ch] */
gint Board[NUMBER_OF_CELLS][NUMBER_OF_CELLS];

/* copy of board on previous step. stupid. temporary. i hope =) */
gint BoardUndo[NUMBER_OF_CELLS][NUMBER_OF_CELLS];

/* next colors... */
gint NextColors[5]={0, 0, 0, 0, 0};

/* copy of next colors on previous step... */
gint NextColorsUndo[5]={0, 0, 0, 0, 0};

/* Backing pixmap for drawing area */
GtkWidget *main_window;
GtkStyle  *style;
GtkWidget *DrawingArea=NULL;

/* Score labels (also used in interface.[ch]) */
GtkWidget *Hi_score_label=NULL,*User_score_label=NULL;

/* This flag shows should be next colors to be shown
 * This variable is also used in preferences.[ch] and interface.c */
gint Show_next_colors=TRUE;

/* This flag indicates wether to show the path of the ball
 * This variable is also used in preferences.[ch] */
gint Show_path=TRUE;

/* This flag indicates wether to show the footprints of the ball
 * This variable is also used in preferences.[ch] */
gint Show_footprints=TRUE;

/* name of default theme. TODO: make "internal" theme, so, when
 * theme name is NULL we can use it... */
gchar Default_theme_name[]="Default";

/* Name of the current theme. Theme_name should point to that,
   and g_free() must be called when changing name and
   Current_theme_name!=NULL */
gchar *Current_theme_name=NULL;

/* This is the name of theme to use */
gchar *Theme_name=Default_theme_name;

/* The next two variables is also used in interface.[ch] */
gint Score=0, Hi_score=0;

void 	show_rules	 	(GtkWidget *widget, gpointer data);
void 	about 		 	(GtkWidget *widget, gpointer data);
gint 	move_ball 		(GtkWidget *widget, gint source_node, gint target_ball);
void 	new_game 	       	(void);
void 	destroy_about_window	(GtkWidget *widget, gpointer data);
void 	undo_move		(GtkWidget *widget, gpointer data);

GtkItemFactory *Menu_items_factory;

static GtkItemFactoryEntry Menu_items[] =
{
	{N_("/Game"), 		      NULL, 	    NULL, 		0, "<Branch>"},
  	{N_("/Game/tear0"), 	      NULL, 	    NULL, 		0, "<Tearoff>"},
  	{N_("/Game/New"), 	      "<control>N", new_game, 		0, "<Item>"},
  	{N_("/Game/sep0"), 	      NULL, 	    NULL, 		0, "<Separator>"},
        {N_("/Game/Hall of fame"),    "<control>H", show_hall_of_fame,  0, "<Item>"},
        {N_("/Game/sep1"), 	      NULL, 	    NULL, 		0, "<Separator>"},
  	{N_("/Game/Quit"), 	      "<control>Q", destroy, 		0, "<Item>"},
  	{N_("/Edit"), 	      	      NULL, 	    NULL, 		0, "<Branch>"},
  	{N_("/Edit/tear1"),           NULL,         NULL, 		0, "<Tearoff>"},
  	{N_("/Edit/Undo"), 	      "<control>U", undo_move, 		0, "<Item>"},
  	{N_("/Settings"), 	      NULL, 	    NULL, 		0, "<Branch>"},
  	{N_("/Settings/tear1"),       NULL,         NULL, 		0, "<Tearoff>"},
  	{N_("/Settings/Preferences"), "<control>P", preferences_dialog, 0, "<Item>"},
  	{N_("/Help"),                 NULL,         NULL, 		0, "<LastBranch>"},
  	{N_("/Help/tear2"),           NULL,         NULL, 		0, "<Tearoff>"},
  	{N_("/Help/Rules"), 	      "F1",         show_rules, 	0, "<Item>"},
  	{N_("/Help/sep2"),  	      NULL,         NULL, 		0, "<Separator>"},
  	{N_("/Help/About"), 	      "<control>A", about, 		0, "<Item>"}
};

static gchar *menu_translate(const gchar *path, gpointer data) {
        gchar *retval=gettext(path);

        return retval;
}

void menu_set_sensitive(GtkItemFactory *ifactory, const gchar *path, gboolean sensitive) {
    GtkWidget *widget;

    g_return_if_fail(ifactory != NULL);

    widget = gtk_item_factory_get_item(ifactory, path);
    gtk_widget_set_sensitive(widget, sensitive);
}

void get_main_menu(GtkWidget *window, GtkWidget **menubar) {
  	gint nmenu_items;

  	nmenu_items=sizeof(Menu_items)/sizeof(Menu_items[0]);
  	Menu_items_factory=gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<Main>", NULL);
  	gtk_item_factory_set_translate_func(Menu_items_factory, menu_translate, NULL, NULL);
  	gtk_item_factory_create_items(Menu_items_factory, nmenu_items, Menu_items, NULL);
  	gtk_window_add_accel_group(GTK_WINDOW(window), Menu_items_factory->accel_group);

  	if(menubar) {
    		*menubar=Menu_items_factory->widget;
        }
}

void undo_move(GtkWidget *widget, gpointer data) {

        memcpy(Board, BoardUndo, sizeof(Board));
        memcpy(NextColors, NextColorsUndo, sizeof(NextColors));
        AnimationInProgress=FALSE;
        menu_set_sensitive(Menu_items_factory, "/Edit/Undo", FALSE);
        draw_board(DrawingArea, Pixmap);
        draw_next_balls();
        gtk_widget_draw(DrawingArea, NULL);
}

gint count_free_cells(void) {
  	gint i,j,counter=0;

  	for(i=0;i<NumberOfXCells;i++)
    		for(j=0;j<NumberOfYCells;j++)
      			if(!Board[i][j])
				counter++;

  	return counter;
}

gint new_turn(gint number) {
  	gint i,j,k=0,free_cells_number;

  	if(!(free_cells_number=count_free_cells())) {
    		return FALSE;
        }

  	k=0;
  	do {
      		i=(gint)(((gfloat)NumberOfXCells)*rand()/(RAND_MAX+1.0));
      		j=(gint)(((gfloat)NumberOfYCells)*rand()/(RAND_MAX+1.0));
      		if (Board[i][j] == 0) {
	  		Board[i][j]=NextColors[k];
	  		k++;
		} else {
			continue;
                }
      		/* I hope that k!=0 */
      		NextColors[k-1]=1+(int)(((float)NUMBER_OF_BALLS)*rand()/(RAND_MAX+1.0));
    	} while(k<((number<free_cells_number) ? number : free_cells_number));

  	return TRUE;
}

/* 
   start a new game:
   clear the game field, create five new balls
   and set score to 0.
 */
void new_game(void){
  	if(AnimationInProgress) {
      		gtk_timeout_remove(TimerTag);
      		AnimationInProgress=FALSE;
    	}

        memset(Board,0,sizeof(Board));

  	new_turn(5);
        menu_set_sensitive(Menu_items_factory, "/Edit/Undo", FALSE);
  	Score=0;
  	set_user_score_label();
  	if(DrawingArea != NULL) {
                draw_next_balls();
      		gtk_widget_draw(DrawingArea, NULL);
    	}
}

/* Show dialog box with game rules*/
void show_rules(GtkWidget *widget, gpointer data) {
  	GtkWidget *window;
  	GtkWidget *vbox,*hbox,*button_box;
  	GtkWidget *label;
  	GtkWidget *separator;
  	GtkWidget *ok_button;

  	window=gtk_widget_new(gtk_window_get_type(),
			      "GtkObject::user_data", NULL,
			      "GtkWindow::type", GTK_WINDOW_TOPLEVEL,
			      "GtkWindow::allow_grow", FALSE,
			      "GtkWindow::allow_shrink", FALSE, NULL);
  	gtk_window_set_title(GTK_WINDOW(window), _("Rules"));
        gtk_window_set_wmclass(GTK_WINDOW(window), "GtkBalls_Rules", "GtkBalls");
  	gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);
  	gtk_container_border_width(GTK_CONTAINER(window), 2);
  
  	vbox=gtk_vbox_new(FALSE, 0);
  	gtk_container_add(GTK_CONTAINER(window), vbox);

  	hbox=gtk_hbox_new(FALSE, 0);
  	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

  	label=gtk_label_new(_("The play area of GtkBalls is a 9x9 grid. When GtkBalls  is\n" \
			      "first  started  a  number  of balls are placed in a random\n" \
			      "manner on the grid. Balls can be in seven different colors\n" \
			      "(red,  green,  blue,  cyan,  brown, pink, yellow). You may\n" \
			      "move balls on the grid by clicking them  and  clicking  an\n" \
			      "empty  square  on the grid as destination for the ball you\n" \
			      "selected. Balls will only move if they are not blocked  by\n" \
			      "other  balls.  Each time you move a ball, unless you score\n" \
			      "some points , new balls appear in a random manner  on  the\n" \
			      "grid.  If  the grid is full then the game is lost. However\n" \
			      "this is bound to happen, your goal is to make the  highest\n" \
			      "score. In order to do this you must make lines in a verti-\n" \
			      "cal, horizontal or diagonal way with  balls  of  the  same\n" \
			      "color.  A  line  must have at least five balls of the same\n" \
			      "color. Making a line of  five  balls  will  earn  you  ten\n" \
			      "points  for  your score. If you make lines over five balls\n" \
			      "it will earn you even more points. In order  to  help  you\n" \
			      "decide which ball you are going to move, there is an indi-\n" \
			      "cator on top of the grid that shows what colors  the  next\n" \
			      "balls that will appear on the grid will be."));
  	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);

  	separator=gtk_hseparator_new();
  	gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 5);

  	button_box=gtk_hbox_new(FALSE, 0);
  	gtk_box_pack_start(GTK_BOX(vbox), button_box, TRUE, TRUE, 2);

  	ok_button=gtk_button_new_with_label(_("OK"));
        GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
  	gtk_signal_connect_object(GTK_OBJECT(ok_button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(window));
  	gtk_box_pack_start(GTK_BOX(button_box), ok_button, TRUE, TRUE, 0);

  	gtk_widget_show_all(window);
        gtk_widget_grab_default(ok_button);
}

/* Show about dialog box */
void about(GtkWidget *widget, gpointer data) {
  	GtkWidget *window;
  	GtkWidget *vbox,*hbox,*buttons_box;
  	GdkPixmap *splash; /* GtkBalls logo */
  	GtkWidget *label; /* Information about author and copying conditions */
  	GtkWidget *pixmap;
  	GdkBitmap *mask;
  	GtkStyle  *style;
  	GtkWidget *separator;
  	GtkWidget *ok_button, *license_button;
  	gchar     strbuf[1024];
  
  	window=gtk_widget_new(gtk_window_get_type(),
			      "GtkObject::user_data", NULL,
			      "GtkWindow::type", GTK_WINDOW_TOPLEVEL,
			      "GtkWindow::allow_grow", FALSE,
			      "GtkWindow::modal", TRUE,
			      "GtkWindow::allow_shrink", FALSE, NULL);
  	gtk_object_set_user_data(GTK_OBJECT(window), "about");
  	gtk_window_set_title(GTK_WINDOW (window), _("About"));
        gtk_window_set_wmclass(GTK_WINDOW(window), "GtkBalls_About", "GtkBalls");
  	gtk_container_border_width(GTK_CONTAINER(window), 2);
  
  	vbox=gtk_vbox_new(FALSE, 0);
  	gtk_container_add(GTK_CONTAINER(window), vbox);

  	hbox=gtk_hbox_new(FALSE, 0);
  	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
  
  	style=gtk_widget_get_style(window);
  	splash=gdk_pixmap_create_from_xpm(DrawingArea->window, &mask,
				          &style->bg[GTK_STATE_NORMAL],
				          DATADIR "/gtkballs/gtkballs-logo.xpm");
  	if(splash) {
		pixmap=gtk_pixmap_new(splash, mask);
  		gtk_box_pack_start(GTK_BOX(hbox), pixmap, TRUE, TRUE, 4);
        }

  	g_snprintf(strbuf, sizeof(strbuf), _("GtkBalls %s\n Copyright (C) 1998-1999 Eugene Morozov\n" \
					     "<jmv@lucifer.dorms.spbu.ru>, <john_morozov@yahoo.com>\n\n" \
		       		       	     "GtkBalls comes with ABSOLUTELY NO WARRANTY;\n" \
		       		       	     "for details press \"Show license\" button.\n" \
      		       		       	     "This is free software and you are welcome to\n" \
				       	     "redistribute it under certain conditions;\n" \
				       	     "press \"Show license\" button for details."),VERSION);
  	label=gtk_label_new(strbuf);
  	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);
  
  	separator=gtk_hseparator_new();
  	gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 5);

  	buttons_box=gtk_hbutton_box_new();
  	gtk_box_pack_start(GTK_BOX(vbox), buttons_box, TRUE, TRUE, 2);
        gtk_button_box_set_layout(GTK_BUTTON_BOX(buttons_box), GTK_BUTTONBOX_SPREAD);


  	ok_button=gtk_button_new_with_label("OK");
        GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
  	gtk_signal_connect(GTK_OBJECT(ok_button), "clicked", GTK_SIGNAL_FUNC(destroy), GTK_OBJECT(window));
  	gtk_box_pack_start(GTK_BOX(buttons_box), ok_button, TRUE, TRUE, 0);

  	license_button=gtk_button_new_with_label(_(" Show license "));
        GTK_WIDGET_SET_FLAGS(license_button, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
  	gtk_signal_connect(GTK_OBJECT(license_button), "clicked", GTK_SIGNAL_FUNC(gnu_license_dialog), window);
  	gtk_box_pack_start(GTK_BOX(buttons_box), license_button, TRUE, TRUE, 0);

  	gtk_widget_show_all(window);
        gtk_widget_grab_default(ok_button);
}

gint delete_event(GtkWidget *widget, gpointer data) {
  	return (FALSE);
}

gint animate_ball(gpointer data) {
  	GtkWidget *widget;
  	int x,y;
  	static int phase = 0;		/* 1 through NUMBER_OF_PHASES */

  	widget=GTK_WIDGET(data);

  	++phase;
  	phase=phase%NUMBER_OF_PHASES;
  	find_x_y_of_the_node(&x, &y, Jumping_ball, NumberOfXCells, NumberOfYCells);
  	if(widget!=NULL) {
    		draw_ball(widget, Jumping_ball_color, Pixmap, x, y,
		   	  (phase>NUMBER_OF_PHASES/2) ? phase+1 : 3-phase);
        }

  	return TRUE;
}

gint button_press_event(GtkWidget *widget, GdkEventButton *event) {
  	gint x,y,xn,yn;
	gint TmpBoardUndo[NUMBER_OF_CELLS][NUMBER_OF_CELLS];
	gint TmpNextColorsUndo[5];

  	if(event->button==1 && Pixmap!=NULL) {
      		x=calculate_board_coord_x(event->x);
      		y=calculate_board_coord_y(event->y);
		find_x_y_of_the_node(&xn, &yn, Jumping_ball, NumberOfXCells, NumberOfYCells);
      		if(Board[x][y]) {
	  		if(AnimationInProgress) {
                                if(x==xn && y==yn) {
	    				return FALSE;
                                } else {
	    				gtk_timeout_remove(TimerTag);
                                }
                        }
	  		/* following line makes sure that ball isn't in the highest position */
	  		draw_ball(widget, Jumping_ball_color, Pixmap, xn, yn, 5);
	  		Jumping_ball=y*NumberOfXCells+x;
	  		Jumping_ball_color=Board[x][y];
	  		AnimationInProgress=TRUE;
	  		TimerTag=gtk_timeout_add(110, animate_ball, widget);
		} else if(AnimationInProgress == TRUE) {
        		memcpy(TmpBoardUndo, Board, sizeof(TmpBoardUndo));
        		memcpy(TmpNextColorsUndo, NextColors, sizeof(TmpNextColorsUndo));
	  		gtk_timeout_remove(TimerTag);
	  		AnimationInProgress=FALSE;
	  		if(!move_ball(widget, Jumping_ball, y*NumberOfXCells+x)) {
	      			TimerTag=gtk_timeout_add(110, animate_ball, widget);
	      			AnimationInProgress=TRUE;
	      			return FALSE;
	    		}
        		memcpy(BoardUndo, TmpBoardUndo, sizeof(BoardUndo));
        		memcpy(NextColorsUndo, TmpNextColorsUndo, sizeof(NextColorsUndo));
        		menu_set_sensitive(Menu_items_factory, "/Edit/Undo", TRUE);
	  		if(!destroy_lines(widget, Pixmap, TRUE)) {
	      			new_turn(3);
                		draw_next_balls();
	    		}
	  		/* if fifth ball will appear, don't increase user's
	     		   score--it's not his merit */
	  		destroy_lines(widget, Pixmap, FALSE);
	  		/* if there's no free cells start a new game */
	  		if(!count_free_cells()) {
	      			if (Score>Scoreboard[9].score) {
					input_name_dialog();
                                }
	      			new_game();
	    		}
	  		draw_board(widget, Pixmap);
	  		gtk_widget_draw(widget, NULL);
		}
    	}

  	return FALSE;
}

void destroy (GtkWidget *widget, gpointer data) {
  	if(data) {
      		gtk_widget_destroy(GTK_WIDGET(data));
      		return;
  	}
  	gtk_main_quit();
}

int main(int argc, char **argv) {
  	GtkWidget *menubar;
  	GtkWidget *vbox,*hbox,*drawing_area_box,*small_balls_box;
  	GdkBitmap *mask;
  	GdkPixmap *icon;
  	gint i;

  	gtk_set_locale();
#ifdef HAVE_NLS
  	bindtextdomain (PACKAGE, LOCALEDIR);
  	textdomain (PACKAGE);
#endif /* HAVE_NLS */
  
  	if(!read_score(Scoreboard)) {
      		strcpy(Scoreboard[0].name, "Eugene");
      		Scoreboard[0].score=100;
      		for(i=1;i<10;i++) {
	  		Scoreboard[i].name[0] = '\0';
	  		Scoreboard[i].score=0;
	  		strcpy(Scoreboard[i].date, "Thu Jul 22 16:40:39 1999");
		}
    	}

  	/* load user's preferences */
  	load_preferences();

  	/* initialize random seed */
  	srand((unsigned int)time(NULL));
        memset(Board,0,sizeof(Board));
  	for(i=0;i<5;i++) {
    		NextColors[i]=1+(gint)(((gfloat)NUMBER_OF_BALLS)*rand()/(RAND_MAX+1.0));
        }
  	new_turn(5);

  	gtk_init(&argc, &argv);

  	main_window=gtk_widget_new(gtk_window_get_type(),
				   "GtkObject::user_data", NULL,
				   "GtkWindow::type", GTK_WINDOW_TOPLEVEL,
				   "GtkWindow::title", "GtkBalls",
				   "GtkWindow::allow_grow", FALSE,
				   "GtkWindow::allow_shrink", FALSE, NULL);
        gtk_window_set_wmclass(GTK_WINDOW(main_window), "GtkBalls_Main", "GtkBalls");
  	style=gtk_widget_get_style(main_window);

  	gtk_signal_connect(GTK_OBJECT(main_window), "delete_event", GTK_SIGNAL_FUNC(delete_event), NULL);
  	gtk_signal_connect(GTK_OBJECT(main_window), "destroy", GTK_SIGNAL_FUNC (destroy), NULL);
  	gtk_widget_realize(main_window);
  	DrawingArea=gtk_drawing_area_new();

  	vbox=gtk_vbox_new(FALSE, 5);
  	hbox=gtk_hbox_new(TRUE, 0);
  	drawing_area_box=gtk_hbox_new(FALSE, 0);
  	small_balls_box=gtk_hbox_new(TRUE, 0);

  	get_main_menu(main_window, &menubar);
        menu_set_sensitive(Menu_items_factory, "/Edit/Undo", FALSE);
  	gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, TRUE, 0);

  	Hi_score=Scoreboard[0].score;
  	Score=0;

  	Hi_score_label=gtk_label_new(_("Hi-score: 0"));
  	set_hi_score_label();
  	gtk_box_pack_start(GTK_BOX(hbox), Hi_score_label, FALSE, FALSE, 0);

  	User_score_label=gtk_label_new(_("Your score: 0"));
  	set_user_score_label();
  	gtk_box_pack_end(GTK_BOX(hbox), User_score_label, FALSE, FALSE, 0);

  	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(hbox), FALSE, FALSE, 0);

  	gtk_signal_connect(GTK_OBJECT(DrawingArea), "expose_event", (GtkSignalFunc)expose_event, NULL);
  	gtk_signal_connect(GTK_OBJECT(DrawingArea), "button_press_event", (GtkSignalFunc)button_press_event, NULL);
  	gtk_widget_set_events(DrawingArea, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);

  	gtk_box_pack_start(GTK_BOX(drawing_area_box), DrawingArea, TRUE, FALSE, 10);
  	gtk_box_pack_start(GTK_BOX(vbox), drawing_area_box, FALSE, FALSE, 10);
  	gtk_container_add(GTK_CONTAINER(main_window), vbox);

  	icon=gdk_pixmap_create_from_xpm(main_window->window, &mask,
				        &style->bg[GTK_STATE_NORMAL],
				        DATADIR "/gtkballs/gtkballs_16x16.xpm");
  	if(icon) {
		gdk_window_set_icon(main_window->window, NULL, icon, mask);
        }
  	if(!load_theme(Theme_name)) {
                g_warning(_("Failed loading theme \"%s\"! Trying \"Default\"\n"),Theme_name);
                Theme_name=Default_theme_name;
  		if(!load_theme(Theme_name)) {
        		g_error(_("Failed loading theme \"%s\"!\n"),Theme_name);
                }
  	}
  	gtk_widget_show_all(main_window);
  	Pixmap=gdk_pixmap_new(DrawingArea->window, NumberOfXCells*CellXSize, NumberOfYCells*CellYSize, -1);

  	Ball1=gtk_pixmap_new(gbSmallBalls[NextColors[0]].pixmap, gbSmallBalls[NextColors[0]].mask);
  	gtk_box_pack_start(GTK_BOX(small_balls_box), Ball1, FALSE, FALSE, 0);
        Ball2 = gtk_pixmap_new (gbSmallBalls[NextColors[1]].pixmap, gbSmallBalls[NextColors[1]].mask);
  	gtk_box_pack_start(GTK_BOX(small_balls_box), Ball2, FALSE, FALSE, 0);
  	Ball3=gtk_pixmap_new(gbSmallBalls[NextColors[2]].pixmap, gbSmallBalls[NextColors[2]].mask);
  	gtk_box_pack_start(GTK_BOX(small_balls_box), Ball3, FALSE, FALSE, 0);
  	gtk_box_pack_start(GTK_BOX(hbox), small_balls_box, TRUE, FALSE, 0);
  	if(Show_next_colors) {
      		gtk_widget_show(Ball1);
      		gtk_widget_show(Ball2);
      		gtk_widget_show(Ball3);
    	}
  	gtk_widget_show(small_balls_box);
  	gtk_main();

  	return 0;
}
