/* GtkBalls
 * Copyright (C) 1998-1999 Eugene V. Morozov
 * Modifyed in 2001 by drF_ckoff
 * $Id: interface.c,v 1.3 1999/12/28 03:33:35 jmv Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <config.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>

#include "gtkballs.h"
#include "interface.h"
#include "scoreboard.h"
#include "preferences.h"
#include "gfx.h"

static GtkWidget *dialog 	= NULL;
static gint 	  saved_score 	= 0;
static gchar 	 *user_name 	= NULL;
static gchar 	  last_player_name[15];

struct point { gint x; gint y; };

void my_usleep(gint x) {
	struct timeval st_delay;
  
  	st_delay.tv_usec=x;
  	st_delay.tv_sec=0;
  	select(32, NULL, NULL, NULL, &st_delay);
}

void init_names_scores_and_dates (GtkWidget **names, GtkWidget **scores, GtkWidget **dates, int pos) {
  	gint i;
  	gchar *str;
  
  	if(!read_score(Scoreboard)) {
      		simple_message_box(_("Unable to read score.\n"), _("OK"));
    	}

  	for (i=0;i<10;i++) {
      		str=g_strdup_printf(" %s ", Scoreboard[i].name);
      		names[i]=gtk_label_new(str);
                g_free(str);
      		if(!Scoreboard[i].score) {
                        scores[i]=gtk_label_new("");
      		} else {
			str=g_strdup_printf(" %d ", Scoreboard[i].score);
                        scores[i]=gtk_label_new(str);
                	g_free(str);
                }
      		str=g_strdup_printf(" %s ", Scoreboard[i].date);
      		dates[i]=gtk_label_new(str);
                g_free(str);
    	}
}

void show_hall_of_fame(GtkWidget *widget, gpointer data) {
  	GtkWidget *hall_of_fame;
  	GtkWidget *frame;
  	GtkWidget *table;
  	GtkWidget *vbox,*button_box;
  	GtkWidget *names[10],*scores[10],*dates[10];
  	GtkWidget *close_button;
  	gint i;

  	hall_of_fame=gtk_widget_new(gtk_window_get_type(),
				    "GtkObject::user_data", NULL,
				    "GtkWindow::title", _("Hall of Fame"),
				    "GtkWindow::allow_grow", FALSE,
				    "GtkWindow::allow_shrink", FALSE,
				    "GtkContainer::border_width", 5, NULL);
        gtk_window_set_wmclass(GTK_WINDOW(hall_of_fame), "GtkBalls_Scores", "GtkBalls");

  	gtk_signal_connect(GTK_OBJECT(hall_of_fame), "delete_event", GTK_SIGNAL_FUNC(delete_event), NULL);
  	gtk_signal_connect(GTK_OBJECT(hall_of_fame), "destroy", GTK_SIGNAL_FUNC(destroy), hall_of_fame);

  	frame=gtk_frame_new(_(" Hall of Fame "));
  	vbox=gtk_vbox_new(FALSE, 0);
  	button_box=gtk_hbox_new(TRUE, 0);
  	gtk_container_border_width(GTK_CONTAINER(vbox), 1);
  	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);

  	table=gtk_table_new(9, 3, FALSE);
  	init_names_scores_and_dates(names, scores, dates, (data!=NULL) ? (gint)data : -1);

  	for(i=0;i<10;i++) {
      		gtk_table_attach_defaults(GTK_TABLE(table), names[i],  0, 1, i, i+1);
      		gtk_table_attach_defaults(GTK_TABLE(table), scores[i], 1, 2, i, i+1);
      		gtk_table_attach_defaults(GTK_TABLE(table), dates[i],  2, 3, i, i+1);
    	}

  	gtk_container_add(GTK_CONTAINER(frame), table);

  	close_button=gtk_button_new_with_label(_("Close"));
        GTK_WIDGET_SET_FLAGS(close_button, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
  	gtk_signal_connect(GTK_OBJECT(close_button), "clicked", GTK_SIGNAL_FUNC(destroy), hall_of_fame);
  	gtk_box_pack_start(GTK_BOX(button_box), close_button, TRUE, TRUE, 0);

  	gtk_box_pack_start(GTK_BOX(vbox), button_box, FALSE, FALSE, 4);

  	gtk_container_add(GTK_CONTAINER(hall_of_fame), vbox);

  	gtk_widget_show_all(hall_of_fame);
  	gtk_grab_add(hall_of_fame);
        gtk_widget_grab_default(close_button);
}

/* shows simple message box */
void simple_message_box(gchar *message, gchar *label) {
  	GtkWidget *dialog;
  	GtkWidget *prompt_label;
  	GtkWidget *button;

  	dialog=gtk_dialog_new();
        gtk_window_set_wmclass(GTK_WINDOW(dialog), "GtkBalls_Dialog", "GtkBalls");
  	gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 10);

  	prompt_label=gtk_label_new(message);
  	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), prompt_label, TRUE, TRUE, 5);

  	button=gtk_button_new_with_label(label);
  	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(destroy), dialog);
  	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
  	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  	gtk_widget_grab_default(button);

  	gtk_widget_show_all(dialog);
  	gtk_grab_add(dialog);
}

/* returns board coordinate of the window position */
gint calculate_board_coord_x(gint window_coord) {
  	return (window_coord-1)/CellXSize;
}

gint calculate_board_coord_y(gint window_coord) {
  	return (window_coord-1)/CellYSize;
}

void update_rectangle(GtkWidget *widget, gint x, gint y, gint w, gint h) {
  	GdkRectangle r;

  	r.x = x;
  	r.y = y;
  	r.width = w;
  	r.height = h;
  	gtk_widget_draw(widget, &r);
}

void set_hi_score_label(void) {
  	gchar *str;

  	str=g_strdup_printf(_("Hi-score: %i"), Hi_score);
  	gtk_label_set(GTK_LABEL(Hi_score_label), str);
        g_free(str);
}

void set_user_score_label(void) {
  	gchar *str;

  	if(Score>Hi_score) {
      		Hi_score=Score;
      		set_hi_score_label();
    	}

  	str=g_strdup_printf(_("Your score: %i"), Score);
  	gtk_label_set(GTK_LABEL(User_score_label), str);
        g_free(str);
}

void add_point_if_first(struct point *balls_to_delete, gint *number_of_balls_to_delete, gint x, gint y) {
  	gint i;

  	for(i=0;i<*number_of_balls_to_delete;i++) {
    		if((balls_to_delete[i].x==x) && (balls_to_delete[i].y==y)) {
      			return;
                }
        }
  	balls_to_delete[*number_of_balls_to_delete].x=x;
  	balls_to_delete[*number_of_balls_to_delete].y=y;
  	(*number_of_balls_to_delete)++;
}

void clear_line(gint x, gint y, gint length, gint direction, struct point *balls_to_delete, gint *number_of_balls_to_delete) {
  	gint i,x1,y1;

  	switch(direction) {
    		case 0:
      			for(i=y-length;i<y;i++) {
				add_point_if_first(balls_to_delete, number_of_balls_to_delete, x, i);
                        }
      			break;
    		case 1:
      			for(i=x-length;i<x;i++) {
				add_point_if_first(balls_to_delete, number_of_balls_to_delete, i, y);
                        }
      			break;
    		case 2:
      			/* save length */
      			i=length;
      			for(x1=x-1,y1=y-1;length>0;x1--,y1--,length--) {
				add_point_if_first(balls_to_delete, number_of_balls_to_delete, x1, y1);
                        }
      			/* restore length */
      			length=i;
      			break;
    		case 3:
      			i=length;
      			for(x1=x-1,y1=y+1;length>0;x1--,y1++,length--) {
				add_point_if_first(balls_to_delete, number_of_balls_to_delete, x1, y1);
                        }
      			length=i;
      			break;
    		case 4:
      			i=length;
      			for(x1=x-1,y1=y-1;length>0;x1--,y1--,length--) {
				add_point_if_first(balls_to_delete, number_of_balls_to_delete, x1, y1);
                        }
      			length=i;
      			break;
    		case 5:
      			i=length;
      			for(x1=x-1,y1=y+1;length>0;x1--,y1++,length--) {
				add_point_if_first(balls_to_delete, number_of_balls_to_delete, x1, y1);
                        }
      			length=i;
      			break;
    	}
}

gint destroy_lines(GtkWidget *widget, GdkPixmap *pixmap, gint count_score) {
  	gint x,y,length;
  	gint x1,y1;
  	gint retval=FALSE;
  	gint number_of_balls_to_delete=0;
  	struct point *balls_to_delete;
  	gint i,color;

  	balls_to_delete=g_malloc(NumberOfXCells*NumberOfYCells*sizeof(struct point));

  	/* test vertical rows */
  	for(x=0;x<NumberOfXCells;x++) {
      		for(length=1,y=1;y<NumberOfYCells-1;y++) {
			if(Board[x][y] && (Board[x][y]==Board[x][y-1])) {
	    			while((Board[x][y]==Board[x][y-1]) && y<NumberOfYCells) {
					length++;
					y++;
	      			}
	    			if(length>4) {
					clear_line(x, y, length, 0, balls_to_delete, &number_of_balls_to_delete);
					retval=TRUE;
	      			}
	    			length=1;
	  		}
                }
    	}

  	/* test horizontal rows */
  	for(y=0;y<NumberOfYCells;y++) {
      		for(length=1,x=1;x<NumberOfXCells-1;x++) {
			if((Board[x][y] && (Board[x][y]==Board[x-1][y]))) {
	    			while((Board[x][y]==Board[x-1][y]) && x<NumberOfXCells) {
					length++;
					x++;
	      			}
	    			if(length>4) {
					clear_line(x, y, length, 1, balls_to_delete, &number_of_balls_to_delete);
					retval=TRUE;
	      			}
	    			length = 1;
	  		}
                }
    	}

  	/* test diagonal rows from the left side to the right */
  	for(y=0;y<NumberOfYCells;y++) {
      		/* from the top to the bottom */
      		for(length=1,x1=1,y1=y+1;y1<NumberOfYCells-1 && x1<NumberOfXCells-1;x1++,y1++) {
			if(Board[x1][y1] && (Board[x1][y1]==Board[x1-1][y1-1])) {
	    			while((Board[x1][y1]==Board[x1-1][y1-1]) && x1<NumberOfXCells && y1<NumberOfYCells) {
					length++;
					x1++;
					y1++;
	      			}
	    			if(length>4) {
					clear_line(x1, y1, length, 2, balls_to_delete, &number_of_balls_to_delete);
					retval=TRUE;
	      			}
	    			length = 1;
	  		}
                }

      		/* from bottom to top */
      		for (length=1,x1=1,y1=y-1;y1>=0 && x1<NumberOfXCells;x1++,y1--) {
			if (Board[x1][y1] && (Board[x1][y1]==Board[x1-1][y1+1])) {
	    			while((Board[x1][y1]==Board[x1-1][y1+1]) && x1<NumberOfXCells && y1>=0) {
					length++;
					x1++;
					y1--;
	      			}
	    			if(length>4) {
					clear_line(x1, y1, length, 3, balls_to_delete, &number_of_balls_to_delete);
					retval=TRUE;
	      			}
	    			length = 1;
	  		}
                }
    	}

  	/* test diagonal rows from the top to the bottom */
  	for(x=0;x<NumberOfXCells;x++) {
      		/* from the top to the bottom */
      		for(length=1,x1=x+1,y1=1;y1<NumberOfYCells-1 && x1<NumberOfXCells-1;x1++,y1++) {
			if(Board[x1][y1] && (Board[x1][y1]==Board[x1-1][y1-1])) {
	    			while((Board[x1][y1]==Board[x1-1][y1-1]) && x1<NumberOfXCells && y1<NumberOfYCells) {
					length++;
					x1++;
					y1++;
	      			}
	    			if(length>4) {
					clear_line(x1, y1, length, 4, balls_to_delete, &number_of_balls_to_delete);
					retval=TRUE;
	      			}
	    			length = 1;
	  		}
                }

      		/* from bottom to top */
      		for(length=1,x1=x+1,y1=NumberOfYCells-2;y1>=0 && x1<NumberOfXCells;x1++,y1--) {
			if(Board[x1][y1] && (Board[x1][y1] == Board[x1-1][y1+1])) {
	    			while((Board[x1][y1]==Board[x1-1][y1+1]) && x1<NumberOfXCells && y1>=0) {
					length++;
					x1++;
					y1--;
	      			}
	    			if(length>4) {
					clear_line(x1, y1, length, 5, balls_to_delete, &number_of_balls_to_delete);
					retval=TRUE;
	      			}
	    			length = 1;
	  		}
                }
    	}

  	if(retval) {
      		/* TODO: redraw only balls we're deleting */
      		for(color=BLUE_BALL;color<=CYAN_BALL;color++) {
	  		for(i=0;i<number_of_balls_to_delete;i++) {
	      			Board[balls_to_delete[i].x][balls_to_delete[i].y]=color;
	      			draw_board(widget, pixmap);
	      			usleep(25000);
	      			gtk_widget_draw(widget, NULL);
	    		}
		}

      		for(i=0;i<number_of_balls_to_delete;i++) {
			Board[balls_to_delete[i].x][balls_to_delete[i].y]=0;
                }

      		draw_board(widget, pixmap);
      		gtk_widget_draw(widget, NULL);

      		if(count_score) {
	  		Score=Score+10+2*(number_of_balls_to_delete-5)*(number_of_balls_to_delete-5);
	  		if (!Show_next_colors) {
	    			Score++;
                        }
	  		set_user_score_label();
		}
    	}

  	g_free(balls_to_delete);
  	return retval;
}

void read_entry(GtkWidget *widget, gpointer data) {
  	struct score_board current_entry;
  	time_t current_time;
  	struct tm *timeptr;
  	gint pos;
  
  	user_name=gtk_entry_get_text(GTK_ENTRY(data));
  	if(!strlen(user_name)) {
    		strcpy(current_entry.name, _("Anonymous"));
  	} else {
    		strcpy(current_entry.name, user_name);
        }
  	strcpy(last_player_name, user_name);
  	current_entry.score=saved_score;
  	current_time=time(NULL);
  	timeptr=localtime(&current_time);
  	if(!timeptr) {
      		simple_message_box(_("Unable to determine current date.\n"), _("OK"));
      		current_entry.date[0]='\0';
    	} else {
      		/* I use _() for format string because some people might want
	 	   format of the date more traditional for their locale */
      		if(!strftime(current_entry.date, 30, _("%a %b %d %H:%M %Y"), timeptr)) {
	  		simple_message_box(_("Unable to determine current date.\n"), _("OK"));
	  		current_entry.date[0] = '\0';
		}
    	}
  
  	if(!read_score(Scoreboard)) {
      		simple_message_box (_("Unable to read score.\n"), _("OK"));
    	}
  	pos=insert_entry_in_score_board(Scoreboard, current_entry);
  	if(!write_score(Scoreboard)) {
      		simple_message_box(_("Unable to save score.\n"), _("OK"));
    	}
  	gtk_grab_remove(dialog);
  	gtk_widget_destroy(dialog);
  	/* show scores to let user see if (s)he's on top ;) */
  	show_hall_of_fame(NULL, (gpointer)pos);
}

gchar *input_name_dialog(void) {
  	GtkWidget *prompt_label;
  	GtkWidget *name;
  	GtkWidget *button;

  	/* we have to save score, because they will be set to 0 in new_game() */
  	saved_score=Score;

  	dialog=gtk_dialog_new();
        gtk_window_set_wmclass(GTK_WINDOW(dialog), "GtkBalls_Inputname", "GtkBalls");

  	gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 2);

  	prompt_label=gtk_label_new(_("Enter your name"));
  	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), prompt_label, TRUE, TRUE, 0);

  	name=gtk_entry_new_with_max_length(14);
  	/* restore the last player's name */
  	if(user_name) {
    		gtk_entry_set_text(GTK_ENTRY(name), last_player_name);
    		gtk_entry_select_region(GTK_ENTRY(name), 0, -1);
  	}
  	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), name, TRUE, TRUE, 5);

  	button=gtk_button_new_with_label(_("OK"));
  	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(read_entry), name);
  	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
  	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  	gtk_widget_grab_default(button);

  	gtk_widget_show_all(dialog);
  	gtk_grab_add (dialog);

  	return NULL;
}
