/* GtkBalls
 * Copyright (C) 1998-1999 Eugene V. Morozov
 * Modifyed in 2001 by drF_ckoff
 * $Id: scoreboard.c,v 1.1.1.1 1999/10/23 20:25:05 jmv Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "gtkballs.h"
#include "scoreboard.h"

#define BUFFER_SIZE 1024

gint write_score(struct score_board *b) {
  	int fd;
  	gint i;
  	gchar *buf;
  	sigset_t sset;
  	struct flock lockinfo;

  	/* block signals before writing, to prevent possible file corruption */
  	sigemptyset(&sset);
  	sigaddset(&sset, SIGHUP);
  	sigaddset(&sset, SIGINT);
  	sigaddset(&sset, SIGQUIT);
  	sigaddset(&sset, SIGTERM);
  	sigprocmask(SIG_BLOCK, &sset, NULL);
  
  	if((fd=open(LOCALSTATEDIR "/gtkballs-scores", O_WRONLY))<0) {
    		/* unable to save score */
    		return FALSE;
        }

  	/* get write lock before writing scores */
  	lockinfo.l_whence=SEEK_SET;
  	lockinfo.l_start=0;
  	lockinfo.l_len=0;
  	i=0;
  	while(1) {
    		lockinfo.l_type=F_WRLCK;
    		if(!fcntl(fd, F_SETLK, &lockinfo)) break;
    		if(i>=3) {
      			close(fd);
      			return FALSE;
    		}
    		i++;
  	}
  
  	for(i=0;i<10;i++) {
    		if(strlen(b[i].name)) {
      			buf=g_strdup_printf("%s\t%i\t%s\n", b[i].name, b[i].score, b[i].date);
    		} else {
      			buf=g_strdup_printf("<none>\t0\t<none>\n");
                }
    		write(fd, buf, strlen(buf));
                g_free(buf);
  	}
  	close(fd);

  	sigprocmask(SIG_UNBLOCK, &sset, NULL);
  	return TRUE;
}

gint read_score(struct score_board *b) {
  	FILE *fp;
  	gchar buffer[BUFFER_SIZE];
  	gchar **str_val;
        gint  i=0,j;
  
  	if(!(fp=fopen(LOCALSTATEDIR "/gtkballs-scores", "r"))) {
		return FALSE;
        }
      	while(i<10 && fgets(buffer, BUFFER_SIZE, fp)) {
        	g_strchomp(buffer);
        	str_val=g_strsplit(buffer,"\t",3);
        	if(str_val[0] && str_val[1] &&str_val[2] && str_val[0][0] && str_val[1][0] && str_val[2][0]) {
                        if(g_strcasecmp(str_val[0],"<none>")) {
                                strncpy(b[i].name, str_val[0], 15);
    				b[i].score=strtol(str_val[1], NULL, 10);
    				if((b[i].score==LONG_MIN) || (b[i].score==LONG_MAX)) {
      					b[i].score=0;
                                }
                                strncpy(b[i].date, str_val[2], 30);
                        }
        	}
                g_strfreev(str_val);
                i++;
        }
        fclose(fp);
  
  	if(i<9) {
    		for(j=i;j<10;j++) {
			b[j].name[0] = '\0';
			b[j].score = 0;
			b[j].date[0] = '\0';
      		}
        }
  	return TRUE;
}

gint insert_entry_in_score_board(struct score_board *board, struct score_board entry) {
  	gint i=0,j;

  	if(entry.score<=0) {
    		return -1;
        }

  	while(i<10 && board[i].score>entry.score) {
    		i++;
        }

  	if(i>9) {
    		return -1;
        }

  	for(j=8;j>=i;j--) {
      		strncpy(board[j+1].name, board[j].name, 15);
      		strncpy(board[j+1].date, board[j].date, 30);
      		board[j+1].score=board[j].score;
    	}

  	strncpy(board[i].name, entry.name, 15);
  	strncpy(board[i].date, entry.date, 30);
  	board[i].score = entry.score;

  	return i;
}
